@@ -136,12 +136,13 @@ class ValueCountTermList : public TermList {
 	return it == spy->values.end();
     }
 
-    Xapian::termcount get_approx_size() const { unsupported_method(); }
-    Xapian::termcount get_wdf() const { unsupported_method(); }
+    Xapian::termcount get_approx_size() const { unsupported_method(); return 0; }
+    Xapian::termcount get_wdf() const { unsupported_method(); return 0; }
     Xapian::PositionIterator positionlist_begin() const {
 	unsupported_method();
+	return Xapian::PositionIterator(NULL);
     }
-    Xapian::termcount positionlist_count() const { unsupported_method(); }
+    Xapian::termcount positionlist_count() const { unsupported_method(); return 0; }
 };
 
 /** A string with a corresponding frequency.
@@ -233,12 +234,13 @@ class StringAndFreqTermList : public TermList {
 	return it == values.end();
     }
 
-    Xapian::termcount get_approx_size() const { unsupported_method(); }
-    Xapian::termcount get_wdf() const { unsupported_method(); }
+    Xapian::termcount get_approx_size() const { unsupported_method(); return 0; }
+    Xapian::termcount get_wdf() const { unsupported_method(); return 0; }
     Xapian::PositionIterator positionlist_begin() const {
 	unsupported_method();
+	return Xapian::PositionIterator(NULL);
     }
-    Xapian::termcount positionlist_count() const { unsupported_method(); }
+    Xapian::termcount positionlist_count() const { unsupported_method(); return 0; }
 };
 
 /** Get the most frequent items from a map from string to frequency.
