@@ -2,7 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2003,2004,2006,2007,2009 Olly Betts
+ * Copyright 2003,2004,2006,2007,2008,2009 Olly Betts
+ * Copyright 2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,12 +24,16 @@
 #include <config.h>
 
 #include <xapian/document.h>
-#include <xapian/types.h>
+
 #include "document.h"
+#include "documentvaluelist.h"
 #include "maptermlist.h"
+#include "serialise.h"
+#include "str.h"
+
 #include <xapian/error.h>
+#include <xapian/types.h>
 #include <xapian/valueiterator.h>
-#include "utils.h"
 
 #include <algorithm>
 #include <string>
@@ -43,28 +48,28 @@ Document::Document(Document::Internal *internal_) : internal(internal_)
 {
 }
 
-Document::Document() : internal(new Xapian::Document::Internal())
+Document::Document() : internal(new Xapian::Document::Internal)
 {
 }
 
 string
 Document::get_value(Xapian::valueno value) const
 {
-    DEBUGAPICALL(string, "Document::get_value", value);
+    LOGCALL(API, string, "Document::get_value", value);
     RETURN(internal->get_value(value));
 }
 
 string
 Document::get_data() const
 {
-    DEBUGAPICALL(string, "Document::get_data", "");
+    LOGCALL(API, string, "Document::get_data", NO_ARGS);
     RETURN(internal->get_data());
 }
 
 void
 Document::set_data(const string &data)
 {
-    DEBUGAPICALL(void, "Document::set_data", data);
+    LOGCALL_VOID(API, "Document::set_data", data);
     internal->set_data(data);
 }
 
@@ -93,21 +98,21 @@ Document::get_description() const
 void
 Document::add_value(Xapian::valueno valueno, const string &value)
 {
-    DEBUGAPICALL(void, "Document::add_value", valueno << ", " << value);
+    LOGCALL_VOID(API, "Document::add_value", valueno | value);
     internal->add_value(valueno, value);
 }
 
 void
 Document::remove_value(Xapian::valueno valueno)
 {
-    DEBUGAPICALL(void, "Document::remove_value", valueno);
+    LOGCALL_VOID(API, "Document::remove_value", valueno);
     internal->remove_value(valueno);
 }
 
 void
 Document::clear_values()
 {
-    DEBUGAPICALL(void, "Document::clear_values", "");
+    LOGCALL_VOID(API, "Document::clear_values", NO_ARGS);
     internal->clear_values();
 }
 
@@ -116,8 +121,7 @@ Document::add_posting(const string & tname,
 			Xapian::termpos tpos,
 			Xapian::termcount wdfinc)
 {
-    DEBUGAPICALL(void, "Document::add_posting",
-		 tname << ", " << tpos << ", " << wdfinc);
+    LOGCALL_VOID(API, "Document::add_posting", tname | tpos | wdfinc);
     if (tname.empty()) {
 	throw InvalidArgumentError("Empty termnames aren't allowed.");
     }
@@ -127,7 +131,7 @@ Document::add_posting(const string & tname,
 void
 Document::add_term(const string & tname, Xapian::termcount wdfinc)
 {
-    DEBUGAPICALL(void, "Document::add_term", tname << ", " << wdfinc);
+    LOGCALL_VOID(API, "Document::add_term", tname | wdfinc);
     if (tname.empty()) {
 	throw InvalidArgumentError("Empty termnames aren't allowed.");
     }
@@ -138,8 +142,7 @@ void
 Document::remove_posting(const string & tname, Xapian::termpos tpos,
 			 Xapian::termcount wdfdec)
 {
-    DEBUGAPICALL(void, "Document::remove_posting",
-		 tname << ", " << tpos << ", " << wdfdec);
+    LOGCALL_VOID(API, "Document::remove_posting", tname | tpos | wdfdec);
     if (tname.empty()) {
 	throw InvalidArgumentError("Empty termnames aren't allowed.");
     }
@@ -149,59 +152,67 @@ Document::remove_posting(const string & tname, Xapian::termpos tpos,
 void
 Document::remove_term(const string & tname)
 {
-    DEBUGAPICALL(void, "Document::remove_term", tname);
+    LOGCALL_VOID(API, "Document::remove_term", tname);
     internal->remove_term(tname);
 }
 
 void
 Document::clear_terms()
 {
-    DEBUGAPICALL(void, "Document::clear_terms", "");
+    LOGCALL_VOID(API, "Document::clear_terms", NO_ARGS);
     internal->clear_terms();
 }
 
 Xapian::termcount
 Document::termlist_count() const {
-    DEBUGAPICALL(Xapian::termcount, "Document::termlist_count", "");
+    LOGCALL(API, Xapian::termcount, "Document::termlist_count", NO_ARGS);
     RETURN(internal->termlist_count());
 }
 
 TermIterator
 Document::termlist_begin() const
 {
-    DEBUGAPICALL(TermIterator, "Document::termlist_begin", "");
+    LOGCALL(API, TermIterator, "Document::termlist_begin", NO_ARGS);
     RETURN(TermIterator(internal->open_term_list()));
 }
 
 Xapian::termcount
 Document::values_count() const {
-    DEBUGAPICALL(Xapian::termcount, "Document::values_count", "");
+    LOGCALL(API, Xapian::termcount, "Document::values_count", NO_ARGS);
     RETURN(internal->values_count());
 }
 
 ValueIterator
 Document::values_begin() const
 {
-    DEBUGAPICALL(ValueIterator, "Document::values_begin", "");
-    // Force the values to be read and cached.
-    internal->need_values();
-    RETURN(ValueIterator(0, *this));
-}
-
-ValueIterator
-Document::values_end() const
-{
-    DEBUGAPICALL(ValueIterator, "Document::values_end", "");
-    RETURN(ValueIterator(internal->values_count(), *this));
+    LOGCALL(API, ValueIterator, "Document::values_begin", NO_ARGS);
+    // Calling values_count() has the side effect of making sure that they have
+    // been read into the std::map "values" member of internal.
+    if (internal->values_count() == 0) RETURN(ValueIterator());
+    RETURN(ValueIterator(new DocumentValueList(internal)));
 }
 
 docid
 Document::get_docid() const
 {
-    DEBUGAPICALL(docid, "Document::get_docid", "");
+    LOGCALL(API, docid, "Document::get_docid", NO_ARGS);
     RETURN(internal->get_docid());
 }
 
+std::string
+Document::serialise() const
+{
+    LOGCALL(API, std::string, "Document::serialise", NO_ARGS);
+    return serialise_document(*this);
+}
+
+Document
+Document::unserialise(const std::string &s)
+{
+    LOGCALL_STATIC(API, Document, "Document::unserialise", s);
+    return unserialise_document(s);
+}
+
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -209,7 +220,7 @@ Document::get_docid() const
 void
 OmDocumentTerm::add_position(Xapian::termpos tpos)
 {
-    DEBUGAPICALL(void, "OmDocumentTerm::add_position", tpos);
+    LOGCALL_VOID(DB, "OmDocumentTerm::add_position", tpos);
 
     // We generally expect term positions to be added in approximately
     // increasing order, so check the end first
@@ -230,14 +241,14 @@ OmDocumentTerm::add_position(Xapian::termpos tpos)
 void
 OmDocumentTerm::remove_position(Xapian::termpos tpos)
 {
-    DEBUGAPICALL(void, "OmDocumentTerm::remove_position", tpos);
+    LOGCALL_VOID(DB, "OmDocumentTerm::remove_position", tpos);
     
     // Search for the position the term occurs at.  Use binary chop to
     // search, since this is a sorted list.
     vector<Xapian::termpos>::iterator i;
     i = lower_bound(positions.begin(), positions.end(), tpos);
     if (i == positions.end() || *i != tpos) {
-	throw Xapian::InvalidArgumentError("Position `" + om_tostring(tpos) +
+	throw Xapian::InvalidArgumentError("Position `" + str(tpos) +
 				     "' not found in list of positions that `" +
 				     tname +
 				     "' occurs at,"
@@ -252,8 +263,8 @@ OmDocumentTerm::get_description() const
     string description;
 
     description = "OmDocumentTerm(" + tname +
-	    ", wdf = " + om_tostring(wdf) +
-	    ", positions[" + om_tostring(positions.size()) + "]" +
+	    ", wdf = " + str(wdf) +
+	    ", positions[" + str(positions.size()) + "]" +
 	    ")";
     return description;
 }
@@ -289,7 +300,7 @@ Xapian::Document::Internal::set_data(const string &data_)
 TermList *
 Xapian::Document::Internal::open_term_list() const
 {
-    DEBUGCALL(MATCH, TermList *, "Document::Internal::open_term_list", "");
+    LOGCALL(MATCH, TermList *, "Document::Internal::open_term_list", NO_ARGS);
     if (terms_here) {
 	RETURN(new MapTermList(terms.begin(), terms.end()));
     }
@@ -308,7 +319,6 @@ Xapian::Document::Internal::add_value(Xapian::valueno valueno, const string &val
 	// removing it.
 	values.erase(valueno);
     }
-    value_nos.clear();
 }
 
 void
@@ -317,19 +327,17 @@ Xapian::Document::Internal::remove_value(Xapian::valueno valueno)
     need_values();
     map<Xapian::valueno, string>::iterator i = values.find(valueno);
     if (i == values.end()) {
-	throw Xapian::InvalidArgumentError("Value #" + om_tostring(valueno) +
+	throw Xapian::InvalidArgumentError("Value #" + str(valueno) +
 		" is not present in document, in "
 		"Xapian::Document::Internal::remove_value()");
     }
     values.erase(i);
-    value_nos.clear();
 }
 
 void
 Xapian::Document::Internal::clear_values()
 {
     values.clear();
-    value_nos.clear();
     values_here = true;
 }
 
@@ -440,7 +448,7 @@ Xapian::Document::Internal::need_terms() const
 Xapian::valueno
 Xapian::Document::Internal::values_count() const
 {
-    DEBUGLINE(UNKNOWN, "Xapian::Document::Internal::values_count() called");
+    LOGLINE(DB, "Xapian::Document::Internal::values_count() called");
     need_values();
     Assert(values_here);
     return values.size();
@@ -455,12 +463,12 @@ Xapian::Document::Internal::get_description() const
 
     if (values_here) {
 	if (data_here) description += ", ";
-	description += "values[" + om_tostring(values.size()) + "]";
+	description += "values[" + str(values.size()) + "]";
     }
 
     if (terms_here) {
 	if (data_here || values_here) description += ", ";
-	description += "terms[" + om_tostring(terms.size()) + "]";
+	description += "terms[" + str(terms.size()) + "]";
     }
 
     if (database.get()) {
@@ -479,8 +487,8 @@ Xapian::Document::Internal::need_values() const
 {
     if (!values_here) {
 	if (database.get()) {
-	    values = do_get_all_values();
-	    value_nos.clear();
+	    Assert(values.empty());
+	    do_get_all_values(values);
 	}
 	values_here = true;
     }
