@@ -3,10 +3,11 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001 Hein Ragas
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011 Olly Betts
  * Copyright 2006,2008 Lemur Consulting Ltd
  * Copyright 2009 Richard Boulton
  * Copyright 2009 Kan-Ru Chen
+ * Copyright 2011 Dan Colish
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -86,22 +87,13 @@ using namespace Xapian;
 // byte in the term).
 #define MAX_SAFE_TERM_LENGTH 245
 
-/** Delete file, throwing an error if we can't delete it (but not if it
- *  doesn't exist).
+/** Maximum number of times to try opening the tables to get them at a
+ *  consistent revision.
+ *
+ *  This is mostly just to avoid any chance of an infinite loop - normally
+ *  we'll either get then on the first or second try.
  */
-static void
-sys_unlink_if_exists(const string & filename)
-{
-#ifdef __WIN32__
-    if (msvc_posix_unlink(filename.c_str()) == -1) {
-#else
-    if (unlink(filename) == -1) {
-#endif
-	if (errno == ENOENT) return;
-	throw Xapian::DatabaseError("Can't delete file: `" + filename + "'",
-				    errno);
-    }
-}
+const int MAX_OPEN_RETRIES = 100;
 
 /* This finds the tables, opens them at consistent revisions, manages
  * determining the current and next revision numbers, and stores handles
@@ -129,12 +121,7 @@ BrassDatabase::BrassDatabase(const string &brass_dir, int action,
 	return;
     }
 
-    const char *p = getenv("XAPIAN_MAX_CHANGESETS");
-    if (p)
-	max_changesets = atoi(p);
-
     if (action != Xapian::DB_OPEN && !database_exists()) {
-	// FIXME: if we allow Xapian::DB_OVERWRITE, check it here
 
 	// Create the directory for the database, if it doesn't exist
 	// already.
@@ -163,7 +150,6 @@ BrassDatabase::BrassDatabase(const string &brass_dir, int action,
 
     get_database_write_lock(false);
     // if we're overwriting, pretend the db doesn't exist
-    // FIXME: if we allow Xapian::DB_OVERWRITE, check it here
     if (action == Xapian::DB_CREATE_OR_OVERWRITE) {
 	create_and_open_tables(block_size);
 	return;
@@ -258,8 +244,7 @@ BrassDatabase::open_tables_consistent()
     value_manager.reset();
 
     bool fully_opened = false;
-    int tries = 100;
-    int tries_left = tries;
+    int tries_left = MAX_OPEN_RETRIES;
     while (!fully_opened && (tries_left--) > 0) {
 	if (spelling_table.open(revision) &&
 	    synonym_table.open(revision) &&
@@ -410,7 +395,15 @@ BrassDatabase::set_revision_number(brass_revision_number_t new_revision)
 
     int changes_fd = -1;
     string changes_name;
-
+    
+    // always check max_changesets for modification since last revision
+    const char *p = getenv("XAPIAN_MAX_CHANGESETS");
+    if (p) {
+	max_changesets = atoi(p);
+    } else {
+	max_changesets = 0;
+    }
+ 
     if (max_changesets > 0) {
 	brass_revision_number_t old_revision = get_revision_number();
 	if (old_revision) {
@@ -439,8 +432,11 @@ BrassDatabase::set_revision_number(brass_revision_number_t new_revision)
 	    pack_uint(buf, old_revision);
 	    pack_uint(buf, new_revision);
 
-	    // FIXME - if DANGEROUS mode is in use, this should be 1 not 0.
-	    pack_uint(buf, 0u); // Changes can be applied to a live database.
+#ifndef DANGEROUS
+	    buf += '\x00'; // Changes can be applied to a live database.
+#else
+	    buf += '\x01';
+#endif
 
 	    io_write(changes_fd, buf.data(), buf.size());
 
@@ -472,11 +468,30 @@ BrassDatabase::set_revision_number(brass_revision_number_t new_revision)
     } catch (...) {
 	// Remove the changeset, if there was one.
 	if (changes_fd >= 0) {
-	    sys_unlink_if_exists(changes_name);
+	    (void)io_unlink(changes_name);
 	}
 
 	throw;
     }
+    
+    // Only remove the oldest_changeset if we successfully write a new changeset and
+    // we have a revision number greater than max_changesets
+    if (changes_fd >= 0 && max_changesets < new_revision) {
+	// use the oldest changeset we know about to begin deleting to the stop_changeset
+	// if nothing went wrong only one file should be deleted, otherwise
+	// attempts will be made to clean up more
+	brass_revision_number_t oldest_changeset = stats.get_oldest_changeset();
+	brass_revision_number_t stop_changeset = new_revision - max_changesets;
+	while (oldest_changeset < stop_changeset) {
+	    if (io_unlink(db_dir + "/changes" + str(oldest_changeset))) {
+		LOGLINE(DB, "Removed changeset " << oldest_changeset);
+	    } else {
+		LOGLINE(DB, "Skipping changeset " << oldest_changeset << 
+			", likely removed before");
+	    }
+	    stats.set_oldest_changeset(oldest_changeset++);
+	}
+    }
 }
 
 void
@@ -543,12 +558,16 @@ BrassDatabase::send_whole_database(RemoteConnection & conn, double end_time)
     filepath += '/';
     for (const char * p = filenames; *p; p += *p + 1) {
 	string leaf(p + 1, size_t(static_cast<unsigned char>(*p)));
-        filepath.replace(db_dir.size() + 1, string::npos, leaf);
-	if (file_exists(filepath)) {
-	    // FIXME - there is a race condition here - the file might get
-	    // deleted between the file_exists() test and the access to send it.
+	filepath.replace(db_dir.size() + 1, string::npos, leaf);
+#ifdef __WIN32__
+	int fd = msvc_posix_open(filepath.c_str(), O_RDONLY);
+#else
+	int fd = open(filepath.c_str(), O_RDONLY);
+#endif
+	if (fd > 0) {
+	    fdcloser closefd(fd);
 	    conn.send_message(REPL_REPLY_DB_FILENAME, leaf, end_time);
-	    conn.send_file(REPL_REPLY_DB_FILEDATA, filepath, end_time);
+	    conn.send_file(REPL_REPLY_DB_FILEDATA, fd, end_time);
 	}
     }
 }
@@ -645,7 +664,14 @@ BrassDatabase::write_changesets_to_fd(int fd,
 
 	    // Look for the changeset for revision start_rev_num.
 	    string changes_name = db_dir + "/changes" + str(start_rev_num);
-	    if (file_exists(changes_name)) {
+#ifdef __WIN32__
+	    int fd_changes = msvc_posix_open(changes_name.c_str(), O_RDONLY);
+#else
+	    int fd_changes = open(changes_name.c_str(), O_RDONLY);
+#endif
+	    if (fd_changes > 0) {
+		fdcloser closefd(fd_changes);
+
 		// Send it, and also update start_rev_num to the new value
 		// specified in the changeset.
 		brass_revision_number_t changeset_start_rev_num;
@@ -659,10 +685,8 @@ BrassDatabase::write_changesets_to_fd(int fd,
 		if (changeset_start_rev_num >= changeset_end_rev_num) {
 		    throw Xapian::DatabaseError("Changeset start revision is not less than end revision");
 		}
-		// FIXME - there is a race condition here - the file might get
-		// deleted between the file_exists() test and the access to
-		// send it.
-		conn.send_file(REPL_REPLY_CHANGESET, changes_name, 0.0);
+
+		conn.send_file(REPL_REPLY_CHANGESET, fd_changes, 0.0);
 		start_rev_num = changeset_end_rev_num;
 		if (info != NULL) {
 		    ++(info->changeset_count);
@@ -1260,10 +1284,12 @@ BrassWritableDatabase::replace_document(Xapian::docid did,
 	    Xapian::Internal::RefCntPtr<const BrassWritableDatabase> ptrtothis(this);
 	    BrassTermList termlist(ptrtothis, did);
 	    Xapian::TermIterator term = document.termlist_begin();
-	    brass_doclen_t new_doclen = termlist.get_doclength();
+	    brass_doclen_t old_doclen = termlist.get_doclength();
+	    stats.delete_document(old_doclen);
+	    brass_doclen_t new_doclen = old_doclen;
+
 	    string old_tname, new_tname;
 
-	    stats.delete_document(new_doclen);
 	    termlist.next();
 	    while (!termlist.at_end() || term != document.termlist_end()) {
 		int cmp;
@@ -1337,7 +1363,8 @@ BrassWritableDatabase::replace_document(Xapian::docid did,
 		termlist_table.set_termlist(did, document, new_doclen);
 
 	    // Set the new document length
-	    inverter.set_doclength(did, new_doclen, false);
+	    if (new_doclen != old_doclen)
+		inverter.set_doclength(did, new_doclen, false);
 	    stats.add_document(new_doclen);
 	}
 
