@@ -1,9 +1,8 @@
 /* flint_values.cc: Values in flint databases
  *
- * ----START-LICENCE----
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005 Olly Betts
+ * Copyright 2002,2003,2004,2005,2008 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,9 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
- * -----END-LICENCE-----
  */
 
 #include <config.h>
@@ -30,13 +28,13 @@
 using std::string;
 using std::make_pair;
 
-#include "omdebug.h"
+#include "debuglog.h"
 
 /** Generate key for document @a docid's values. */
 inline void
 make_key(string & key, Xapian::docid did)
 {
-    DEBUGCALL_STATIC(DB, void, "make_key", key << ", " << did);
+    LOGCALL_STATIC_VOID(DB, "make_key", key | did);
     key = flint_docid_to_key(did);
 }
 
@@ -46,20 +44,19 @@ FlintValueTable::unpack_entry(const char ** pos,
 				 Xapian::valueno * this_value_no,
 				 string & this_value)
 {
-    DEBUGCALL_STATIC(DB, void, "FlintValueTable::unpack_entry",
-		     "[pos], [end], " << this_value_no << ", " << this_value);
-    if (!unpack_uint(pos, end, this_value_no)) {
+    LOGCALL_STATIC_VOID(DB, "FlintValueTable::unpack_entry", pos | (void*)end | this_value_no | this_value);
+    if (!F_unpack_uint(pos, end, this_value_no)) {
 	if (*pos == 0) throw Xapian::DatabaseCorruptError("Incomplete item in value table");
 	else throw Xapian::RangeError("Value number in value table is too large");
     }
 
-    if (!unpack_string(pos, end, this_value)) {
+    if (!F_unpack_string(pos, end, this_value)) {
 	if (*pos == 0) throw Xapian::DatabaseCorruptError("Incomplete item in value table");
 	else throw Xapian::RangeError("Item in value table is too large");
     }
 
-    DEBUGLINE(DB, "FlintValueTable::unpack_entry(): value no " <<
-	      this_value_no << " is `" << this_value << "'");
+    LOGLINE(DB, "FlintValueTable::unpack_entry(): value no " <<
+	    this_value_no << " is `" << this_value << "'");
 }
 
 void
@@ -67,10 +64,10 @@ FlintValueTable::encode_values(string & s,
 			       Xapian::ValueIterator it,
 			       const Xapian::ValueIterator & end)
 {
-    DEBUGCALL(DB, void, "FlintValueTable::encode_values", "[&s], " << it << ", " << end);
+    LOGCALL_VOID(DB, "FlintValueTable::encode_values", s | it | end);
     while (it != end) {
-	s += pack_uint(it.get_valueno());
-	s += pack_string(*it);
+	s += F_pack_uint(it.get_valueno());
+	s += F_pack_string(*it);
 	++it;
     }
 }
@@ -78,7 +75,7 @@ FlintValueTable::encode_values(string & s,
 void
 FlintValueTable::set_encoded_values(Xapian::docid did, const string & enc)
 {
-    DEBUGCALL(DB, void, "FlintValueTable::set_encoded_values", did << ", " << enc);
+    LOGCALL_VOID(DB, "FlintValueTable::set_encoded_values", did | enc);
     string key;
     make_key(key, did);
     add(key, enc);
@@ -89,7 +86,7 @@ FlintValueTable::get_value(string & value,
 			      Xapian::docid did,
 			      Xapian::valueno valueno) const
 {
-    DEBUGCALL(DB, void, "FlintValueTable::get_value", value << ", " << did << ", " << valueno);
+    LOGCALL_VOID(DB, "FlintValueTable::get_value", value | did | valueno);
     string key;
     make_key(key, did);
     string tag;
@@ -121,7 +118,7 @@ void
 FlintValueTable::get_all_values(map<Xapian::valueno, string> & values,
 				   Xapian::docid did) const
 {
-    DEBUGCALL(DB, void, "FlintValueTable::get_all_values", "[values], " << did);
+    LOGCALL_VOID(DB, "FlintValueTable::get_all_values", values | did);
     string key;
     make_key(key, did);
     string tag;
@@ -145,7 +142,7 @@ FlintValueTable::get_all_values(map<Xapian::valueno, string> & values,
 void
 FlintValueTable::delete_all_values(Xapian::docid did)
 {
-    DEBUGCALL(DB, void, "FlintValueTable::delete_all_values", did);
+    LOGCALL_VOID(DB, "FlintValueTable::delete_all_values", did);
     string key;
     make_key(key, did);
     del(key);
