@@ -4,6 +4,7 @@
  * Copyright 2001,2002 Ananova Ltd
  * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
  * Copyright 2009 Lemur Consulting Ltd
+ * Copyright 2011 Action Without Borders
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -179,8 +180,9 @@ class Enquire::Internal : public Xapian::Internal::RefCntBase {
 		      const RSet *omrset,
 		      const MatchDecider *mdecider,
 		      const MatchDecider *matchspy_legacy) const;
+
 	ESet get_eset(Xapian::termcount maxitems, const RSet & omrset, int flags,
-		      double k, const ExpandDecider *edecider) const;
+		      double k, const ExpandDecider *edecider, Xapian::weight min_wt) const;
 
 	TermIterator get_matching_terms(Xapian::docid did) const;
 	TermIterator get_matching_terms(const Xapian::MSetIterator &it) const;
