@@ -3,8 +3,9 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2011 Olly Betts
  * Copyright 2009 Lemur Consulting Ltd
+ * Copyright 2011 Action Without Borders
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -965,6 +966,8 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	 *		     query.
 	 *
 	 *  @exception Xapian::InvalidArgumentError  See class documentation.
+	 *
+	 *  @{
 	 */
 	MSet get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		      Xapian::doccount checkatleast = 0,
@@ -981,6 +984,7 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 		      const MatchDecider * mdecider = 0) const {
 	    return get_mset(first, maxitems, 0, omrset, mdecider);
 	}
+	/** @} */
 
 	static const int INCLUDE_QUERY_TERMS = 1;
 	static const int USE_EXACT_TERMFREQ = 2;
@@ -1002,6 +1006,8 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	 *  @param edecider  a decision functor to use to decide whether a
 	 *		     given term should be put in the ESet
 	 *
+	 *  @param min_wt    the minimum weight for included terms
+	 *
 	 *  @return	     An ESet object containing the results of the
 	 *		     expand.
 	 *
@@ -1031,6 +1037,37 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	    return get_eset(maxitems, omrset, 0, 1.0, edecider);
 	}
 
+	/** Get the expand set for the given rset.
+	 *
+	 *  @param maxitems  the maximum number of items to return.
+	 *  @param omrset    the relevance set to use when performing
+	 *		     the expand operation.
+	 *  @param flags     zero or more of these values |-ed together:
+	 *		      - Xapian::Enquire::INCLUDE_QUERY_TERMS query
+	 *			terms may be returned from expand
+	 *		      - Xapian::Enquire::USE_EXACT_TERMFREQ for multi
+	 *			dbs, calculate the exact termfreq; otherwise an
+	 *			approximation is used which can greatly improve
+	 *			efficiency, but still returns good results.
+	 *  @param k	     the parameter k in the query expansion algorithm
+	 *		     (default is 1.0)
+	 *  @param edecider  a decision functor to use to decide whether a
+	 *		     given term should be put in the ESet
+	 *
+	 *  @param min_wt    the minimum weight for included terms
+	 *
+	 *  @return	     An ESet object containing the results of the
+	 *		     expand.
+	 *
+	 *  @exception Xapian::InvalidArgumentError  See class documentation.
+	 */
+	ESet get_eset(Xapian::termcount maxitems,
+			const RSet & omrset,
+			int flags,
+			double k,
+			const Xapian::ExpandDecider * edecider,
+			Xapian::weight min_wt) const;
+
 	/** Get terms which match a given document, by document id.
 	 *
 	 *  This method returns the terms in the current query which match
