@@ -3,6 +3,7 @@
  * ----START-LICENCE----
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2003,2004,2005 Olly Betts
+ * Copyright 2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,9 +17,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
- * -----END-LICENCE-----
  */
 
 #ifndef OM_HGUARD_PHRASEPOSTLIST_H
@@ -54,6 +54,9 @@ class NearPostList : public SelectPostList {
 	    return source->get_termfreq_est() / 2;
 	}
 
+	TermFreqs get_termfreq_est_using_stats(
+            const Xapian::Weight::Internal & stats) const;
+
         NearPostList(PostList *source_, Xapian::termpos window_,
 		     std::vector<PostList *> terms_)
 		: SelectPostList(source_)
@@ -88,6 +91,9 @@ class PhrasePostList : public SelectPostList {
 	    return source->get_termfreq_est() / 3;
 	}
 
+	TermFreqs get_termfreq_est_using_stats(
+            const Xapian::Weight::Internal & stats) const;
+
         PhrasePostList(PostList *source_, Xapian::termpos window_,
 		       std::vector<PostList *> terms_) : SelectPostList(source_)
         {
@@ -97,4 +103,3 @@ class PhrasePostList : public SelectPostList {
 };
 
 #endif /* OM_HGUARD_PHRASEPOSTLIST_H */
-
