@@ -116,6 +116,12 @@ QueryOptimiser::do_subquery(const Xapian::Query::Internal * query, double factor
 	    Xapian::valueno valno(query->parameter);
 	    const string & range_begin = query->tname;
 	    const string & range_end = query->str_parameter;
+	    const string & lb = db.get_value_lower_bound(valno);
+	    if (!lb.empty() &&
+		(range_end < lb ||
+		 range_begin > db.get_value_upper_bound(valno))) {
+		RETURN(new EmptyPostList);
+	    }
 	    RETURN(new ValueRangePostList(&db, valno, range_begin, range_end));
 	}
 
@@ -124,6 +130,11 @@ QueryOptimiser::do_subquery(const Xapian::Query::Internal * query, double factor
 		++total_subqs;
 	    Xapian::valueno valno(query->parameter);
 	    const string & range_begin = query->tname;
+	    const string & lb = db.get_value_lower_bound(valno);
+	    if (!lb.empty() &&
+		range_begin > db.get_value_upper_bound(valno)) {
+		RETURN(new EmptyPostList);
+	    }
 	    RETURN(new ValueGePostList(&db, valno, range_begin));
 	}
 
@@ -132,6 +143,9 @@ QueryOptimiser::do_subquery(const Xapian::Query::Internal * query, double factor
 		++total_subqs;
 	    Xapian::valueno valno(query->parameter);
 	    const string & range_end = query->tname;
+	    if (range_end < db.get_value_lower_bound(valno)) {
+		RETURN(new EmptyPostList);
+	    }
 	    RETURN(new ValueRangePostList(&db, valno, "", range_end));
 	}
 
@@ -274,7 +288,7 @@ struct CmpMaxOrTerms {
 	if (a->get_termfreq_max() == 0) return false;
 	if (b->get_termfreq_max() == 0) return true;
 
-#if (defined(__i386__) && !defined(__SSE2_MATH__)) || defined(__mc68000__)
+#if (defined(__i386__) && !defined(__SSE2_MATH__)) || defined(__mc68000__) || defined(__mc68010__) || defined(__mc68020__) || defined(__mc68030__)
 	// On some architectures, most common of which is x86, floating point
 	// values are calculated and stored in registers with excess precision.
 	// If the two get_maxweight() calls below return identical values in a
@@ -286,6 +300,10 @@ struct CmpMaxOrTerms {
 	// required by nth_element().  This can have serious consequences (e.g.
 	// segfaults).
 	//
+	// Note that m68k only has excess precision in earlier models - 68040
+	// and later are OK:
+	// http://gcc.gnu.org/ml/gcc-patches/2008-11/msg00105.html
+	//
 	// To avoid this, we store each result in a volatile double prior to
 	// comparing them.  This means that the result of this test should
 	// match that on other architectures with the same double format (which
