@@ -1,7 +1,7 @@
 /** @file  remoteconnection.cc
  *  @brief RemoteConnection class used by the remote backend.
  */
-/* Copyright (C) 2006,2007,2008,2009,2010 Olly Betts
+/* Copyright (C) 2006,2007,2008,2009,2010,2011 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -315,26 +315,18 @@ RemoteConnection::send_message(char type, const string &message,
 }
 
 void
-RemoteConnection::send_file(char type, const string &file, double end_time)
+RemoteConnection::send_file(char type, int fd, double end_time)
 {
-    LOGCALL_VOID(REMOTE, "RemoteConnection::send_file", type | file | end_time);
+    LOGCALL_VOID(REMOTE, "RemoteConnection::send_file", type | fd | end_time);
     if (fdout == -1) {
 	throw Xapian::DatabaseError("Database has been closed");
     }
 
-#ifdef __WIN32__
-    int fd = msvc_posix_open(file.c_str(), O_RDONLY);
-#else
-    int fd = open(file.c_str(), O_RDONLY);
-#endif
-    if (fd == -1) throw Xapian::NetworkError("File not found: " + file, errno);
-    fdcloser closefd(fd);
-
     off_t size;
     {
 	struct stat sb;
 	if (fstat(fd, &sb) == -1)
-	    throw Xapian::NetworkError("Couldn't stat file: " + file, errno);
+	    throw Xapian::NetworkError("Couldn't stat file to send", errno);
 	size = sb.st_size;
     }
     // FIXME: Use sendfile() or similar if available?
