@@ -1008,10 +1008,10 @@ phrased_term:
 			    Parse(pParser, ADJ, new Term(width), &state);
 			    goto just_had_operator;
 			}
+		    } else {
+			Parse(pParser, ADJ, NULL, &state);
+			goto just_had_operator;
 		    }
-
-		    Parse(pParser, ADJ, NULL, &state);
-		    goto just_had_operator;
 		}
 	    } else if (op.size() == 2) {
 		if (op == "OR") {
@@ -1031,10 +1031,10 @@ phrased_term:
 			    Parse(pParser, NEAR, new Term(width), &state);
 			    goto just_had_operator;
 			}
+		    } else {
+			Parse(pParser, NEAR, NULL, &state);
+			goto just_had_operator;
 		    }
-
-		    Parse(pParser, NEAR, NULL, &state);
-		    goto just_had_operator;
 		}
 	    }
 	}
@@ -2022,14 +2022,14 @@ delete (yypminor->yy12);
     case 36: /* near_expr */
     case 37: /* adj_expr */
 {
-#line 1917 "queryparser/queryparser.lemony"
+#line 1916 "queryparser/queryparser.lemony"
 (yypminor->yy46)->destroy();
 #line 2028 "queryparser/queryparser_internal.cc"
 }
       break;
     case 35: /* group */
 {
-#line 1951 "queryparser/queryparser.lemony"
+#line 1950 "queryparser/queryparser.lemony"
 (yypminor->yy76)->destroy();
 #line 2035 "queryparser/queryparser_internal.cc"
 }
@@ -2646,93 +2646,91 @@ static void yy_reduce(
 #line 2647 "queryparser/queryparser_internal.cc"
         break;
       case 37: /* compound_term ::= group */
-#line 1895 "queryparser/queryparser.lemony"
-{
-    yygotominor.yy73 = yymsp[0].minor.yy76->as_group(state);
-}
-#line 2654 "queryparser/queryparser_internal.cc"
+#line 1896 "queryparser/queryparser.lemony"
+{ yygotominor.yy73 = yymsp[0].minor.yy76->as_group(state); }
+#line 2652 "queryparser/queryparser_internal.cc"
         break;
       case 38: /* compound_term ::= near_expr */
-#line 1900 "queryparser/queryparser.lemony"
+#line 1899 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[0].minor.yy46->as_near_query(); }
-#line 2659 "queryparser/queryparser_internal.cc"
+#line 2657 "queryparser/queryparser_internal.cc"
         break;
       case 39: /* compound_term ::= adj_expr */
-#line 1903 "queryparser/queryparser.lemony"
+#line 1902 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[0].minor.yy46->as_adj_query(); }
-#line 2664 "queryparser/queryparser_internal.cc"
+#line 2662 "queryparser/queryparser_internal.cc"
         break;
       case 40: /* compound_term ::= BRA expr KET */
-#line 1906 "queryparser/queryparser.lemony"
+#line 1905 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[-1].minor.yy73;   yy_destructor(yypParser,20,&yymsp[-2].minor);
   yy_destructor(yypParser,21,&yymsp[0].minor);
 }
-#line 2671 "queryparser/queryparser_internal.cc"
+#line 2669 "queryparser/queryparser_internal.cc"
         break;
       case 41: /* compound_term ::= SYNONYM TERM */
-#line 1908 "queryparser/queryparser.lemony"
+#line 1907 "queryparser/queryparser.lemony"
 {
     yygotominor.yy73 = new Query(yymsp[0].minor.yy0->get_query_with_synonyms());
     delete yymsp[0].minor.yy0;
   yy_destructor(yypParser,11,&yymsp[-1].minor);
 }
-#line 2680 "queryparser/queryparser_internal.cc"
+#line 2678 "queryparser/queryparser_internal.cc"
         break;
       case 42: /* phrase ::= TERM */
-#line 1919 "queryparser/queryparser.lemony"
+#line 1918 "queryparser/queryparser.lemony"
 {
     yygotominor.yy46 = new Terms;
     yygotominor.yy46->add_positional_term(yymsp[0].minor.yy0);
 }
-#line 2688 "queryparser/queryparser_internal.cc"
+#line 2686 "queryparser/queryparser_internal.cc"
         break;
       case 43: /* phrase ::= phrase TERM */
       case 45: /* phrased_term ::= phrased_term PHR_TERM */ yytestcase(yyruleno==45);
-#line 1924 "queryparser/queryparser.lemony"
+#line 1923 "queryparser/queryparser.lemony"
 {
     yygotominor.yy46 = yymsp[-1].minor.yy46;
     yygotominor.yy46->add_positional_term(yymsp[0].minor.yy0);
 }
-#line 2697 "queryparser/queryparser_internal.cc"
+#line 2695 "queryparser/queryparser_internal.cc"
         break;
       case 44: /* phrased_term ::= TERM PHR_TERM */
-#line 1936 "queryparser/queryparser.lemony"
+#line 1935 "queryparser/queryparser.lemony"
 {
     yygotominor.yy46 = new Terms;
     yygotominor.yy46->add_positional_term(yymsp[-1].minor.yy0);
     yygotominor.yy46->add_positional_term(yymsp[0].minor.yy0);
 }
-#line 2706 "queryparser/queryparser_internal.cc"
+#line 2704 "queryparser/queryparser_internal.cc"
         break;
       case 46: /* group ::= TERM GROUP_TERM */
-#line 1953 "queryparser/queryparser.lemony"
+#line 1952 "queryparser/queryparser.lemony"
 {
     yygotominor.yy76 = new TermGroup;
     yygotominor.yy76->add_term(yymsp[-1].minor.yy0);
     yygotominor.yy76->add_term(yymsp[0].minor.yy0);
 }
-#line 2715 "queryparser/queryparser_internal.cc"
+#line 2713 "queryparser/queryparser_internal.cc"
         break;
       case 47: /* group ::= group GROUP_TERM */
-#line 1959 "queryparser/queryparser.lemony"
+#line 1958 "queryparser/queryparser.lemony"
 {
     yygotominor.yy76 = yymsp[-1].minor.yy76;
     yygotominor.yy76->add_term(yymsp[0].minor.yy0);
 }
-#line 2723 "queryparser/queryparser_internal.cc"
+#line 2721 "queryparser/queryparser_internal.cc"
         break;
       case 48: /* group ::= group EMPTY_GROUP_OK */
-#line 1964 "queryparser/queryparser.lemony"
+#line 1963 "queryparser/queryparser.lemony"
 {
     yygotominor.yy76 = yymsp[-1].minor.yy76;
     yygotominor.yy76->set_empty_ok();
   yy_destructor(yypParser,22,&yymsp[0].minor);
 }
-#line 2732 "queryparser/queryparser_internal.cc"
+#line 2730 "queryparser/queryparser_internal.cc"
         break;
       case 49: /* near_expr ::= TERM NEAR TERM */
       case 51: /* adj_expr ::= TERM ADJ TERM */ yytestcase(yyruleno==51);
-#line 1975 "queryparser/queryparser.lemony"
+#line 1974 "queryparser/queryparser.lemony"
 {
     yygotominor.yy46 = new Terms;
     yygotominor.yy46->add_positional_term(yymsp[-2].minor.yy0);
@@ -2742,11 +2740,11 @@ static void yy_reduce(
 	delete yymsp[-1].minor.yy0;
     }
 }
-#line 2746 "queryparser/queryparser_internal.cc"
+#line 2744 "queryparser/queryparser_internal.cc"
         break;
       case 50: /* near_expr ::= near_expr NEAR TERM */
       case 52: /* adj_expr ::= adj_expr ADJ TERM */ yytestcase(yyruleno==52);
-#line 1985 "queryparser/queryparser.lemony"
+#line 1984 "queryparser/queryparser.lemony"
 {
     yygotominor.yy46 = yymsp[-2].minor.yy46;
     yygotominor.yy46->add_positional_term(yymsp[0].minor.yy0);
@@ -2755,7 +2753,7 @@ static void yy_reduce(
 	delete yymsp[-1].minor.yy0;
     }
 }
-#line 2759 "queryparser/queryparser_internal.cc"
+#line 2757 "queryparser/queryparser_internal.cc"
         break;
       default:
         break;
@@ -2788,7 +2786,7 @@ static void yy_parse_failed(
 
     // If we've not already set an error message, set a default one.
     if (!state->error) state->error = "parse error";
-#line 2792 "queryparser/queryparser_internal.cc"
+#line 2790 "queryparser/queryparser_internal.cc"
   ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
 #endif /* YYNOERRORRECOVERY */
@@ -2808,7 +2806,7 @@ static void yy_syntax_error(
 #line 1554 "queryparser/queryparser.lemony"
 
     yy_parse_failed(yypParser);
-#line 2812 "queryparser/queryparser_internal.cc"
+#line 2810 "queryparser/queryparser_internal.cc"
   ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
 
