@@ -2,8 +2,9 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011 Olly Betts
  * Copyright 2006,2008 Lemur Consulting Ltd
+ * Copyright 2011 Action Without Borders
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -421,6 +422,60 @@ DEFINE_TESTCASE(expandweights2, backend) {
     return true;
 }
 
+DEFINE_TESTCASE(expandweights3, backend) {
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    enquire.set_query(Xapian::Query("this"));
+
+    Xapian::MSet mymset = enquire.get_mset(0, 10);
+
+    Xapian::RSet myrset;
+    Xapian::MSetIterator i = mymset.begin();
+    myrset.add_document(*i);
+    myrset.add_document(*(++i));
+
+    // Set min_wt to 0
+    Xapian::ESet eset = enquire.get_eset(50, myrset, 0, 1.0, 0, 0);
+    if (!startswith(get_dbtype(), "multi")) {
+	// For a single database, the weights should be the same with or
+	// without USE_EXACT_TERMFREQ.
+	TEST_EQUAL_DOUBLE(eset[0].get_weight(), 6.08904001099445);
+	TEST_EQUAL_DOUBLE(eset[1].get_weight(), 6.08904001099445);
+	TEST_EQUAL_DOUBLE(eset[2].get_weight(), 4.73383620844021);
+    } else {
+	// For multiple databases, we expect that using USE_EXACT_TERMFREQ
+	// will result in different weights in some cases.
+	TEST_NOT_EQUAL_DOUBLE(eset[0].get_weight(), 6.08904001099445);
+	TEST_EQUAL_DOUBLE(eset[1].get_weight(), 6.08904001099445);
+	TEST_NOT_EQUAL_DOUBLE(eset[2].get_weight(), 4.73383620844021);
+    }
+    TEST_REL(eset.back().get_weight(),>=,0);
+
+    return true;
+}
+
+
+// tests that negative weights are returned
+DEFINE_TESTCASE(expandweights4, backend) {
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    enquire.set_query(Xapian::Query("paragraph"));
+
+    Xapian::MSet mymset = enquire.get_mset(0, 10);
+
+    Xapian::RSet myrset;
+    Xapian::MSetIterator i = mymset.begin();
+    myrset.add_document(*i);
+    myrset.add_document(*(++i));
+
+    Xapian::ESet eset = enquire.get_eset(37, myrset, 0, 1.0, 0, -100);
+    // Now include negative weights
+    TEST_EQUAL(eset.size(), 37);
+    TEST_REL(eset[36].get_weight(),<,0);
+    TEST_REL(eset[36].get_weight(),>=,-100);
+
+    return true;
+}
+
+
 // tests that when specifying maxitems to get_eset, no more than
 // that are returned.
 DEFINE_TESTCASE(expandmaxitems1, backend) {
@@ -1750,12 +1805,12 @@ DEFINE_TESTCASE(spaceterms1, backend) {
     for (m = mymset.begin(); m != mymset.end(); ++m) ++count;
     TEST_EQUAL(count, 1);
 
-    for (Xapian::valueno value_no = 1; value_no < 7; ++value_no) {
+    for (Xapian::valueno value_no = 0; value_no < 7; ++value_no) {
 	string value = mymset.begin().get_document().get_value(value_no);
 	TEST_NOT_EQUAL(value, "");
 	if (value_no == 0) {
 	    TEST(value.size() > 262);
-	    TEST_EQUAL(static_cast<unsigned char>(value[261]), 255);
+	    TEST_EQUAL(static_cast<unsigned char>(value[262]), 255);
 	}
     }
 
